@@ -49,13 +49,30 @@
 (defvar exwm-floating--cursor-bottom-left nil)
 (defvar exwm-floating--cursor-left nil)
 
-(defvar exwm-workspace--current)
-(defvar exwm-workspace--list)
-(defvar exwm-workspace-current-index)
-(defvar exwm-workspace--switch-history-outdated)
+(defun exwm-floating--set-allowed-actions (id tilling)
+  "Set _NET_WM_ALLOWED_ACTIONS."
+  (xcb:+request exwm--connection
+      (make-instance 'xcb:ewmh:set-_NET_WM_ALLOWED_ACTIONS
+                     :window id
+                     :data (if tilling
+                               (vector xcb:Atom:_NET_WM_ACTION_MINIMIZE
+                                       xcb:Atom:_NET_WM_ACTION_FULLSCREEN
+                                       xcb:Atom:_NET_WM_ACTION_CHANGE_DESKTOP
+                                       xcb:Atom:_NET_WM_ACTION_CLOSE)
+                             (vector xcb:Atom:_NET_WM_ACTION_MOVE
+                                     xcb:Atom:_NET_WM_ACTION_RESIZE
+                                     xcb:Atom:_NET_WM_ACTION_MINIMIZE
+                                     xcb:Atom:_NET_WM_ACTION_FULLSCREEN
+                                     xcb:Atom:_NET_WM_ACTION_CHANGE_DESKTOP
+                                     xcb:Atom:_NET_WM_ACTION_CLOSE)))))
 
-(declare-function exwm-layout--refresh "exwm-layout.el")
-(declare-function exwm-layout--show "exwm-layout.el")
+(defvar exwm-workspace--current)
+(defvar exwm-workspace--struts)
+
+(declare-function exwm-layout--refresh "exwm-layout.el" ())
+(declare-function exwm-layout--show "exwm-layout.el" (id &optional window))
+(declare-function exwm-layout--iconic-state-p "exwm-layout.el" (&optional id))
+(declare-function exwm-workspace--minibuffer-own-frame-p "exwm-workspace.el")
 
 (defun exwm-floating--set-floating (id)
   "Make window ID floating."
@@ -80,8 +97,7 @@
                      (height . ,window-min-height)
                      (unsplittable . t))))) ;and fix the size later
          (outer-id (string-to-number (frame-parameter frame 'outer-window-id)))
-         (container (with-current-buffer (exwm--id->buffer id)
-                      exwm--container))
+         (container (buffer-local-value 'exwm--container (exwm--id->buffer id)))
          (frame-container (xcb:generate-id exwm--connection))
          (window (frame-first-window frame)) ;and it's the only window
          (x (slot-value exwm--geometry 'x))
@@ -205,6 +221,7 @@
                                              xcb:ConfigWindow:Y))
                        :x (- x exwm-floating-border-width)
                        :y (- y exwm-floating-border-width)))
+    (exwm-floating--set-allowed-actions id nil)
     (xcb:flush exwm--connection)
     ;; Set window/buffer
     (with-current-buffer (exwm--id->buffer id)
@@ -217,9 +234,24 @@
       (add-hook 'window-configuration-change-hook #'exwm-layout--refresh)
       (set-window-dedicated-p window t)
       (exwm-layout--show id window))
-    (with-selected-frame exwm-workspace--current
-      (exwm-layout--refresh))
-    (select-frame-set-input-focus frame))
+    (if (exwm-layout--iconic-state-p id)
+        ;; Hide iconic floating X windows.
+        (with-current-buffer (exwm--id->buffer id)
+          (exwm-floating-hide))
+      (with-selected-frame exwm-workspace--current
+        (exwm-layout--refresh))
+      (select-frame-set-input-focus frame))
+    ;; FIXME: Strangely, the Emacs frame can move itself at this point
+    ;;        when there are left/top struts set.  Force resetting its
+    ;;        position seems working, but it'd better to figure out why.
+    (when exwm-workspace--struts
+      (xcb:+request exwm--connection
+          (make-instance 'xcb:ConfigureWindow
+                         :window outer-id
+                         :value-mask (logior xcb:ConfigWindow:X
+                                             xcb:ConfigWindow:Y)
+                         :x 0 :y 0))
+      (xcb:flush exwm--connection)))
   (run-hooks 'exwm-floating-setup-hook)
   ;; Redraw the frame.
   (redisplay))
@@ -269,6 +301,7 @@
                          :sibling (frame-parameter exwm-workspace--current
                                                    'exwm-container)
                          :stack-mode xcb:StackMode:Above)))
+    (exwm-floating--set-allowed-actions id t)
     (xcb:flush exwm--connection)
     (with-current-buffer buffer
       (when exwm--floating-frame        ;from floating to non-floating
@@ -278,9 +311,11 @@
       (setq window-size-fixed nil
             exwm--floating-frame nil
             exwm--frame exwm-workspace--current))
-    (let ((window (frame-selected-window exwm-workspace--current)))
-      (set-window-buffer window buffer)
-      (select-window window)))
+    (unless (exwm-layout--iconic-state-p)
+      ;; Only show X windows in normal state.
+      (let ((window (frame-selected-window exwm-workspace--current)))
+        (set-window-buffer window buffer)
+        (select-window window))))
   (run-hooks 'exwm-floating-exit-hook))
 
 ;;;###autoload
@@ -292,6 +327,8 @@
         (exwm-floating--unset-floating exwm--id)
       (exwm-floating--set-floating exwm--id))))
 
+(declare-function exwm-layout--set-state "exwm-layout.el" (id state))
+
 ;;;###autoload
 (defun exwm-floating-hide ()
   "Hide the current floating X window (which would show again when selected)."
@@ -304,7 +341,7 @@
                        :window exwm--container
                        :value-mask xcb:ConfigWindow:StackMode
                        :stack-mode xcb:StackMode:Below))
-    ;; FIXME: Should it be put into iconic state?
+    (exwm-layout--set-state exwm--id xcb:icccm:WM_STATE:IconicState)
     (xcb:flush exwm--connection)
     (select-frame-set-input-focus exwm-workspace--current)))
 
@@ -487,8 +524,8 @@
              ;; Unmanaged.
              (eq major-mode 'exwm-mode))
     (let ((edges (window-inside-absolute-pixel-edges (frame-selected-window)))
-          (id (with-current-buffer (window-buffer (frame-selected-window))
-                exwm--id)))
+          (id (buffer-local-value 'exwm--id
+                                  (window-buffer (frame-selected-window)))))
       (xcb:+request exwm--connection
           (make-instance 'xcb:SendEvent
                          :propagate 0 :destination id
@@ -533,7 +570,7 @@
       (setq container-or-id
             (if (bufferp buffer-or-id)
                 ;; Managed.
-                (with-current-buffer buffer-or-id exwm--container)
+                (buffer-local-value 'exwm--container buffer-or-id)
               ;; Unmanaged.
               buffer-or-id))
       (xcb:+request exwm--connection
